package cn.iocoder.yudao.module.customer.dal.dataobject.customer;

import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import io.swagger.annotations.ApiModelProperty;
import lombok.*;

import java.math.BigDecimal;
import java.util.Date;

/**
 * 客户 DO
 *
 * @author yanghao
 */
@TableName("ecw_customer")
@Data
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class CustomerDO extends BaseDO {

    @TableId
    private Long id;

    /**
     * 客户编号
     */
    private String number;
    /**
     * 客户名称
     */
    private String name;

    /**
     * 客户英文名称
     */
    private String nameEn;


    /**
     * 付款人姓名
     */
    private String payerName;
    /**
     * 客户等级
     * <p>
     * 枚举 {@link TODO customer_level 对应的类}
     */
    private Integer level;

    /**
     * 信用等级id
     */
    private Integer creditLevel;
    /**
     * 国家
     * <p>
     * 枚举 {@link TODO common_status 对应的类}
     */
    private Long country;
    /**
     * 客户类型
     * <p>
     * 枚举 {@link TODO customer_type 对应的类}
     */
    private String type;


    /**
     * 货运类型 （有海运/空运，或两个都存在） 1,2
     * <p>
     * 枚举 {@link customer_transport_type 对应的类}
     */
    private String transportType;

    /**
     * 客户资源类型  开发客户/公司客户
     * <p>
     * 枚举 {@link customer_resource_type 对应的类}
     */
    private Integer resourceType;
    /**
     * 所属代理
     * <p>
     * 枚举 {@link TODO common_status 对应的类}
     */
    private Long agentId;
    /**
     * 公司名称
     */
    private String company;
    /**
     * 公司名称英文
     */
    private String companyEn;
    /**
     * 联系地址
     */
    private String address;
    /**
     * 产品类型
     */
    private Long productType;
    /**
     * 产品id
     */
    private String productIds;
    /**
     * 常用提货网点
     */
    private String pickupPoints;
    /**
     * 关联会员账号
     */
    private Long memberId;
    /**
     * 客户生日
     */
    private Date birthday;
    /**
     * 结算方式
     * <p>
     * 枚举 {@link TODO customer_balance 对应的类}
     */
    private Integer balance;
    /**
     * 客户来源
     * <p>
     * 枚举 {@link TODO customer_source 对应的类}
     */
    private Integer source;
    /**
     * 图片
     */
    private String picture;
    /**
     * 跟进客服
     * <p>
     * 枚举 {@link TODO common_status 对应的类}
     */
    private Long customerService;


    /**
     * 跟进客服分配时间
     */
    private Date customerServiceAssignedTime;
    /**
     * 跟进客户是否确认接收
     */
    private Boolean isCustomerServiceConfirmed;
    /**
     * 推介人
     */
    private Long promoter;
    /**
     * 客户状态
     * <p>
     * 枚举 {@link TODO customer_status 对应的类}
     */
    private Integer status;

    /**
     * 客户状态更新时间
     */
    private Date statusUpdateTime;
    /**
     * 创建人
     */
    private Long founder;
    /**
     * 部门
     */
    private String department;
    /**
     * 发票抬头
     */
    private String invoiceTitle;
    /**
     * 纳税人识别号
     */
    private String licenseNumber;
    /**
     * 开户行
     */
    private String bank;
    /**
     * 账户
     */
    private String bankNumber;
    /**
     * 项目
     */
    private String project;
    /**
     * 开票地址
     */
    private String billingAddress;
    /**
     * 开票电话
     */
    private String billingTell;
    /**
     * 税率
     */
    private BigDecimal taxRate;
    /**
     * 完善时间
     */
    private Date perfectTime;
    /**
     * 备注
     */
    private String remarks;
    /**
     * 管理员id
     */
    private Long adminId;
    /**
     * 审批人id
     */
    private Long approver;
    /**
     * 通知人id
     */
    private Long notifier;
    /**
     * 其他审批id
     */
    private Long otherApprovalId;
    /**
     * 到仓确认
     */
    private Integer arrivalConfirm;
    /**
     * 重货标准(CBM)
     */
    private BigDecimal weightUnit;
    /**
     * 泡货标准(kg)
     */
    private BigDecimal lightUnit;
    /**
     * 重货标准修改人
     */
    private String weightEditWho;
    /**
     * 重货标准修时间
     */
    private Date weightEditTime;


    /**
     * 是否系统创建 0为手动 1为注册时系统创建 2为下单时系统创建
     */
    private Integer isAuto;

    /**
     * 询价详情
     */
    private String enquiryInfo;


    /**
     * 是否显示提单价格
     */
    private Boolean isShowTidanPrice;


    /**
     * 是否是公海
     */
    private Boolean isInOpenSea;

    /**
     * 进入公海时间
     */
    private Date enterOpenSeaTime;

    /**
     * 预计进入公海时间
     */
    private Date estimateEnterOpenSeaTime;

    /**
     * 延期x月
     */
    private Integer postponeMonth;

    /**
     * 从公海池捞取时间
     */
    private Date catchTime;


    /**
     * vip等级分
     */

    private Long vipLevelScore;

    /**
     * 信用等级分
     */
    private Long creditLevelScore;

    /**
     * 身份证名称
     */
    private String carName;

    /**
     * 身份证号码
     */
    private String carNo;


    /**
     * 是否被订单收货人同步的
     */
    private Boolean isWebOrderConsigneeSync;

    /**
     * 收货人首次分配的发货人的客户经理
     */
    private Long consigneeFirstCustomerService;


    /**
     * 是否新客户
     */
    private Boolean isNew;


    /**
     * 报价单延期次数
     */
    private Integer createOfferPostponeNum;


    /**
     * 询盘信息
     */
    private String inquiry;

    /**
     * 是否潜在客户
     */
    private Boolean isPotential;

    /**
     * 默认付款，1-是，0-否
     */
    private Boolean defaultPay ;

    /**
     * 控货无收货人,1-是，0-否
     */
    private Boolean noConsignee ;

    @ApiModelProperty("延期申请次数")
    private int delayApprovalNum ;


    // ======== 20240821 add start
    /**
     * 首次成交时间
     */
    private Date firstDealTime;

    /**
     * 获取方式
     * <p>
     * 枚举 {@link TODO customer_get_method 字典}
     */
    private Integer getMethod;

    /**
     * 创建入口
     * <p>
     * 枚举 {@link TODO customer_create_from 字典}
     */
    private Integer createFrom;

    /**
     * 归属时间
     */
    private Date customerServiceConfirmedTime;

    /**
     * 客户角色
     * <p>
     * 枚举 {@link TODO customer_role 字典}
     */
    private String roles;


    /**
     * 业务国家 逗号分隔
     */
    private String busiCountryIds;


    /**
     * 竞争对手ID, 关联 customer_competitor 表主键
     */
    private String competitorIds;

    /**
     * 年度发货量
     */
    private BigDecimal weightYearly;

    /**
     * 年度发货次数
     */
    private Integer numYearly;


    /**
     * 默认开票，1-是，0-否
     */
    private Boolean defaultBilling;

    // ======== 20240821 add end

    /**
     * vip等级-中文
     */
    @TableField(exist = false)
    private String vipLevelNameZh;

    /**
     * vip等级-英文
     */
    @TableField(exist = false)
    private String vipLevelNameEn;


    /**
     * 信用等级-中文
     */
    @ApiModelProperty(value = "")
    @TableField(exist = false)
    private String creditLevelNameZh;

    /**
     * 信用等级-英文
     */
    @TableField(exist = false)
    private String creditLevelNameEn;


    /**
     * 主联系人id
     */
    @TableField(exist = false)
    private String defaultContactId;

    /**
     * 主联系人
     */
    @TableField(exist = false)
    private String defaultContactName;

    /**
     *  主联系电话
     *  不加区号
     */
    @TableField(exist = false)
    private String contactPhone ;
    /**
     * 主联系电话
     */
    @TableField(exist = false)
    private String defaultContactPhone;

    /**
     * 主邮箱
     */
    @TableField(exist = false)
    private String defaultEmail;

    /**
     * 主社交方式
     */
    @TableField(exist = false)
    private Integer defaultSocial;

    /**
     * 主社交软件号码
     */
    @TableField(exist = false)
    private String defaultSocialNumber;

    /**
     * 跟进客服名字
     */
    @TableField(exist = false)
    private String customerServiceName;


    /**
     * 国家名称
     */
    @TableField(exist = false)
    private String countryName;


}
