package cn.iocoder.yudao.module.customer.dal.mysql.customer;

import cn.iocoder.yudao.framework.mybatis.core.mapper.BaseMapperX;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.CustomerDO;
import cn.iocoder.yudao.module.customer.dto.CustomerExportReqDTO;
import cn.iocoder.yudao.module.customer.dto.CustomerOrderQueryDTO;
import cn.iocoder.yudao.module.customer.dto.CustomerPageReqDTO;
import cn.iocoder.yudao.module.customer.vo.customer.vo.CustomerDetailRespVO;
import cn.iocoder.yudao.module.customer.vo.customer.vo.CustomerSelectReqVO;
import cn.iocoder.yudao.module.customer.vo.customerDetail.CustomerOrderBackPageVO;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Constants;
import org.apache.ibatis.annotations.*;

import java.util.Date;
import java.util.List;

/**
 * 客户 Mapper
 *
 * @author yanghao
 */
@Mapper
public interface CustomerMapper extends BaseMapperX<CustomerDO> {

    //合并操作日志
    void doMergeCustomerOperateLog(long customer_id1,long customer_id2);

    //合并客户信用日志
    void doMergeCustomerCreditLog(long customer_id1,long customer_id2);

    //合并客户登记日志
    void doMergeCustomerLevelLog(long customer_id1,long customer_id2);

    IPage<CustomerDO> getPage(IPage<CustomerDO> page,
                              @Param(Constants.WRAPPER) Wrapper<CustomerDO> queryWrapper);

    List<CustomerDO> getMyPage(@Param("start") int start, @Param("size") int size, @Param("query")CustomerPageReqDTO customerPageReqDTO);

    List<CustomerDO> getWaitToAssignedCustomerPageReq(@Param("start") int start, @Param("size") int size, @Param("query")CustomerPageReqDTO customerPageReqDTO);
    /**
     * 分页列表统计total
     * @param pageReqVO
     * @return
     */
    long getWaitToAssignedCustomerPageReqCount( @Param("query") CustomerPageReqDTO pageReqVO);
    /**
     * 待分配客户列表导出, 用于 Excel 导出 支持多筛选
     * @param
     * @return
     */
    List<CustomerDO> getWaitToAssignedCustomerListReq( @Param("query")CustomerExportReqDTO customerPageReqDTO);


    long getMyPageCount( @Param("query") CustomerPageReqDTO pageReqVO);

    List<CustomerDO> getCustomerExcelVoList(@Param(Constants.WRAPPER) Wrapper<CustomerDO> queryWrapper);

    List<CustomerDO> getMyCustomerExcelVoList( @Param("query") CustomerExportReqDTO pageReqVO);

    List<CustomerDO> getList(@Param(Constants.WRAPPER) Wrapper<CustomerDO> queryWrapper);

    List<CustomerDO> searchCustomer(@Param("reqVO") CustomerSelectReqVO customerSelectReqVO);

    @ResultType(CustomerDetailRespVO.class)
    @Select({
            "<script>",
                "select c.*,  ",
                    "cl.name_zh as vip_level_name_zh, cl.name_en as vip_level_name_en, ",
                    "",
                    "cc.name_zh as credit_level_name_zh, cc.name_en as credit_level_name_en,",
                    "",
                    "country.name_zh as country_name_zh, country.name_en as country_name_en,",
                    "",
                    "pt.title_zh as product_type_name_zh, pt.title_en as product_type_name_en,",
                    "",
                    "(SELECT GROUP_CONCAT(title_zh) FROM ecw_product p WHERE FIND_IN_SET(p.id,(SELECT product_ids FROM ecw_customer WHERE id=c.id))) AS product_name_zh, ",
                    "(SELECT GROUP_CONCAT(title_en) FROM ecw_product p WHERE FIND_IN_SET(p.id,(SELECT product_ids FROM ecw_customer WHERE id=c.id))) AS product_name_en, ",
                    "",
                    "(SELECT GROUP_CONCAT(title_zh) FROM ecw_node n WHERE FIND_IN_SET(n.id,(SELECT pickup_points FROM ecw_customer WHERE id=c.id))) AS pickup_point_name_zh,",
                    "(SELECT GROUP_CONCAT(title_en) FROM ecw_node n WHERE FIND_IN_SET(n.id,(SELECT pickup_points FROM ecw_customer WHERE id=c.id))) AS pickup_point_name_en,",
                    "",
                    "(SELECT GROUP_CONCAT(name_zh) FROM ecw_country ct WHERE FIND_IN_SET(ct.id,(SELECT busi_country_ids FROM ecw_customer WHERE id=c.id))) AS busi_country_name_zh,",
                    "(SELECT GROUP_CONCAT(name_en) FROM ecw_country ct WHERE FIND_IN_SET(ct.id,(SELECT busi_country_ids FROM ecw_customer WHERE id=c.id))) AS busi_country_name_en,",
                    "",
                    "(SELECT GROUP_CONCAT(name) FROM ecw_customer_competitor ccp WHERE FIND_IN_SET(ccp.id,(SELECT competitor_ids FROM ecw_customer WHERE id=c.id))) AS competitor_names,",
                    "",
                    "uc.nickname as creator_name,",
                    "uu.nickname as updater_name,",
                    "",
                    "c.customer_service,",
                    "u.nickname as customer_service_name",
                    "",
                "from ecw_customer c",
                "left join ecw_customer_level cl on c.level = cl.id",
                "left join ecw_customer_credit cc on c.credit_level = cc.id",
                "left join ecw_country country on c.country = country.id",
                "left join ecw_product_type pt on c.product_type = pt.id",
                "left join system_user u on c.customer_service = u.id",
                "left join system_user uc on c.creator = uc.id",
                "left join system_user uu on c.updater = uu.id",
                "where c.id = #{id}",
            "</script>"
    })
    CustomerDetailRespVO getCustomerDetailRespVO(@Param("id") Long id);


    List<CustomerDO> getMyCustomerList(@Param("loginUserId") Long loginUserId);


    @Select("select distinct a.* from ecw_customer a " +
            "inner join ecw_customer_contacts b on a.id = b.customer_id and b.deleted=0 " +
            "inner join member_user c on b.userid = c.id " +
            "where c.id = #{memberUserId} order by a.id desc limit 1")
    CustomerDO getByMemberUserId(@Param("memberUserId") Long memberUserId);


  /*  @Select({
            "<script>",
            "SELECT " +
            " customer.number, " +
            " customer.`name`, " +
            " customer.`level`, " +
            " customer.`status`, " +
            " customer.create_time, " +
            " customer.company, " +
            " customer.address, " +
            " customer.department, " +
            " cc.`name` as default_contact_name, " +
            " cc.`phone_new` as default_contact_phone, " +
            " (select c.name_zh from ecw_country c where c.id= customer.country) as country_name, " +
            " (select u.nickname from system_user u where u.id = customer.customer_service) as customer_service_name " +
            " FROM ecw_customer  customer " +
            " left join  (SELECT  *  FROM ecw_customer_contacts  WHERE id IN ( SELECT max( s.id ) FROM ecw_customer_contacts s WHERE s.is_default = 1 and s.deleted = 0 GROUP BY s.customer_id ))  cc ",
            " on customer.id = cc.customer_id ",
            " WHERE customer.deleted = 0 ",
            "<when test = 'query.number != null'>",
            "and customer.number like concat('%',#{query.number},'%') ",
            "</when>",
            "<when test = 'query.name != null'>",
            "and customer.name like concat('%',#{query.name},'%') ",
            "</when>",
            "order by customer.id ",
            "</script>"
    })
    List<CustomerExcelVO> getCustomerExcelVoList(@Param("query") CustomerExportReqVO query);

   */


    List<CustomerDO> getToOpenSeaTimeAfterLimitDayCustomerList(@Param("limitDay") Integer limitDay);

    void enterToOpenSea(@Param("customerList") List<CustomerDO> customerList);


    void assignedCustomer(@Param("customerList") List<CustomerDO> customerList, @Param("customerServiceId") Long customerServiceId);

    List<CustomerDO> getWaitConfirmUserToWaitAssigned(@Param("hours") Integer hours);
    List<CustomerDO> getWebConsigneeCustomerToWaitAssigned(@Param("hours") Integer hours);
    void moveToWaitAssigned(@Param("customerList") List<CustomerDO> customerList);

    void catchCustomer(@Param("loginUserId") Long loginUserId,
                       @Param("estimateEnterOpenSeaTime") Date estimateEnterOpenSeaTime,
                       @Param("customerId") Long customerId
    );

    // @ResultType(CustomerCountryVO.class)
    @Select({
            "<script>",
            "select max(replace(a.number, #{nameShort}, '') + 0) as max_number ",
            "from ecw_customer a ",
            "where a.number is not null and a.number like concat(#{nameShort}, '%')",
            "</script>"
    })
    Long getCustomerTmpMaxNumber(String nameShort);


    // @ResultType(CustomerCountryVO.class)
    @Select({
            "<script>",
            "select max(replace(a.number, #{nameShort}, '') + 0) as max_number ",
            "from ecw_customer a inner join ecw_country b on a.country = b.id ",
            "where a.number is not null and a.number like concat(#{nameShort}, '%') and b.name_short = #{nameShort}",
            "</script>"
    })
    Long getCustomerCountryMaxNumber(String nameShort);


    @Select("select * from ecw_customer where length(number) = 10")
    List<CustomerDO> getOldCustomerNumberList();


    @Update("update ecw_customer set estimate_enter_open_sea_time = null, enter_open_sea_time = null where id = #{customerId}")
    void updateCustomerEstimateEnterOpenSeaTimeToNull(@Param("customerId") Long customerId);

    @Select("select * from ecw_customer a where status < 3 and (exists(select 1 from ecw_order_consignee nee where nee.customer_id = a.id) or exists(select 1 from ecw_order_consignor nor where nor.customer_id = a.id))")
    List<CustomerDO> getHasOrderCustomerList();


    void updateCustomerIsNewToOldByTargetLog(@Param("customerId") Long customerId);

    @Update("update ecw_customer set weight_unit = null where id = #{customerId}")
    void updateWeightUnitToNull(Long customerId);

    @Update("update ecw_customer set light_unit = null where id = #{customerId}")
    void updateLightUnitToNull(Long customerId);


    // 修改客户经理下所有成交客户为老客户，先查询写操作日志，再更新。
    @Select("select id from ecw_customer a where deleted = 0 and is_new = 1 and status = 3 and customer_service = #{customerServiceId}")
    List<Long> getNewDealCustomerIdList(Long customerService);
    @Update("update ecw_customer set is_new = 0 where deleted = 0 and is_new = 1 and status = 3 and customer_service = #{customerServiceId}")
    void updateCustomerToOldByCustomerService(@Param("customerServiceId") Long customerServiceId);


    // 更新客户业务类型
    @Update("update ecw_customer set is_new = #{isNew} where deleted = 0 and is_new != #{isNew} and id = #{customerId}")
    void updateCustomerNewOrOld(@Param("customerId") Long customerId, @Param("isNew") Boolean isNew);

    @Update("update ecw_customer set delay_approval_num = #{delayApprovalNum} where deleted = 0  and id = #{customerId}")
    void updateCustomerDelayApprovalNum(@Param("customerId") Long customerId,@Param("delayApprovalNum") int delayApprovalNum);

    List<CustomerOrderBackPageVO> orderList(@Param("start") int start, @Param("size") int size, @Param("query") CustomerOrderQueryDTO query);


    List<CustomerDO> getWaitToConfirmCustomerPageReq(@Param("start") int start, @Param("size") int size, @Param("query") CustomerPageReqDTO pageReqVO);

    long getWaitToConfirmCustomerPageReqCount( @Param("query")  CustomerPageReqDTO pageReqVO);

    List<CustomerDO> getWaitToConfirmCustomerExcelVoListReq(@Param("query") CustomerExportReqDTO reqVO);
    //公海客户列表
    List<CustomerDO> getOpenSeaCustomerPageReq(@Param("start") int start, @Param("size") int size, @Param("query") CustomerPageReqDTO pageReqVO);
    //公海客户列表统计total
    long getOpenSeaCustomerPageReqCount( @Param("query")  CustomerPageReqDTO pageReqVO);

    List<CustomerDO> getPublicCustomerExcelVoReqList(@Param("query") CustomerExportReqDTO reqVO);

    List<CustomerDO> getDeptCustomerPageReq(@Param("start") int start, @Param("size") int size, @Param("query")  CustomerPageReqDTO pageReqVO,@Param("deptId")  Long deptId);

    long getDeptCustomerPageReqCount(@Param("query")  CustomerPageReqDTO pageReqVO,@Param("deptId")  Long deptId);



    List<CustomerDO> getDeptCustomerExcelVoListReq(@Param("query") CustomerExportReqDTO exportReqVO,@Param("deptId")  Long deptId);
}
