package cn.iocoder.yudao.module.customer.dto;

import cn.iocoder.yudao.framework.common.pojo.PageParam;
import com.fasterxml.jackson.annotation.JsonIgnore;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.ToString;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;
import java.util.List;

import static cn.iocoder.yudao.framework.common.util.date.DateUtils.FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND;

@ApiModel("管理后台 - 客户分页 Request DTO")
@Data
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
public class CustomerPageReqDTO extends PageParam {

    @ApiModelProperty(value = "客户编号")
    private String number;

    @ApiModelProperty(value = "客户名称")
    private String name;

    @ApiModelProperty(value = "客户等级")
    private List<Integer> level;

    @ApiModelProperty(value = "客户来源")
    private List<Integer> source;

    @ApiModelProperty(value = "客户类别")
    private List<String> type;

    @ApiModelProperty(value = "客户货运类型", notes = "参见customer_transport_type数据字典")
    private String transportType;

    @ApiModelProperty(value = "客户资源类型", notes = "参见customer_resource_type数据字典")
    private List<Integer> resourceType;

    @ApiModelProperty(value = "市场获客", notes = "参见customer_market_type数据字典")
    private List<Integer> marketType;


    @ApiModelProperty(value = "跟进客服")
    private List<Long> customerService;

    @ApiModelProperty(value = "客户状态")
    private List<Integer> status;

    @ApiModelProperty(value = "部门")
    private String department;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "开始创建时间")
    private Date beginCreateTime;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "结束创建时间")
    private Date endCreateTime;



    @ApiModelProperty(value = "主联系人")
    private String defaultContactName;

    @ApiModelProperty(value = "主联系电话")
    private String defaultContactPhone;



    @ApiModelProperty(value = "信用等级")
    private List<Integer> creditLevel;

    @ApiModelProperty(value = "国家")
    private List<Integer> country;

    //api不显示此3个字段
    //是否接收，后台使用
    @JsonIgnore
    private Boolean isCustomerServiceConfirmed;

    //因接口公用方法新增：24小时内数据查询条件标识
    private String flag4QueryCondition;

    @ApiModelProperty(value = "是否在公海")
    private Boolean isInOpenSea;

    @ApiModelProperty(value = "是否潜在客户")
    private Boolean isPotential;

    @ApiModelProperty(value = "会员编号")
    private String memberCode;
    @ApiModelProperty(value = "会员名称")
    private String memberName;
    @ApiModelProperty(value = "会员手机")
    private String memberMobile;
    @ApiModelProperty(value = "会员区号")
    private String memberAreaCode;
}
