package cn.iocoder.yudao.module.customer.service.customer;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.CustomerDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerContacts.CustomerContactsDO;
import cn.iocoder.yudao.module.customer.dto.CustomerDelayApprovalInfoDto;
import cn.iocoder.yudao.module.customer.dto.CustomerExportReqDTO;
import cn.iocoder.yudao.module.customer.dto.CustomerHandoverApprovalInfoDto;
import cn.iocoder.yudao.module.customer.dto.CustomerPageReqDTO;
import cn.iocoder.yudao.module.customer.vo.customer.vo.*;
import cn.iocoder.yudao.module.ecw.enums.CustomerOperateTypeEnum;
import cn.iocoder.yudao.module.ecw.enums.CustomerStatusEnum;
import org.apache.ibatis.annotations.Param;

import javax.validation.Valid;
import java.util.Collection;
import java.util.Date;
import java.util.List;

/**
 * 客户 Service 接口
 *
 * @author yanghao
 */
public interface CustomerService extends IService<CustomerDO> {

    //合并操作日志
    void doMergeCustomerOperateLog(long customer_id1,long customer_id2);

    //合并客户信用日志
    void doMergeCustomerCreditLog(long customer_id1,long customer_id2);

    void doMergeCustomerLevelLog(long customer_id1,long customer_id2);

    /**
     * 新建订单，收货人同步到后台，如果手机号不存在新增一个客户档案，进入待分配客户
     *
     * @param name
     * @param areaCode
     * @param phone
     * @param company
     * @param email
     * @param consignorCustomerDO
     * @param orderNo
     * @return
     */
    CustomerContactsDO createConsigeeCustomer(String name, String areaCode, String phone, String company, String email, CustomerDO consignorCustomerDO, String orderNo);


    /**
     * 创建客户
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createCustomer(@Valid CustomerCreateReqVO createReqVO);

    /**
     * 更新客户
     *
     * @param updateReqVO 更新信息
     */
    void updateCustomer(@Valid CustomerUpdateReqVO updateReqVO);

    /**
     * 删除客户
     *
     * @param id 编号
     */
    void deleteCustomer(Long id);

    /**
     * 获得客户
     *
     * @param id 编号
     * @return 客户
     */
    CustomerDO getCustomer(Long id);


    CustomerDetailRespVO getCustomerDetailRespVO(Long id);

    /**
     * 获得客户列表
     *
     * @param ids 编号
     * @return 客户列表
     */
    List<CustomerDO> getCustomerList(Collection<Long> ids);

    ///**
    // * 获得自己跟进的客户列表
    // *
    // * @return 客户列表
    // * @param customerService
    // */
    //List<CustomerDO> getMyCustomerList(Long customerService);

    /**
     * 获得客户分页
     *
     * @param pageReqVO 分页查询
     * @return 客户分页
     */
    PageResult<CustomerDO> getCustomerPage(CustomerPageReqVO pageReqVO);

    /**
     * 获取我的客户分页 ，入参支持多选
     * @param pageReqVO
     * @param page
     * @return
     */
    PageResult<CustomerDO> getMyCustomerPage(CustomerPageReqDTO pageReqVO, PageVO page);

    /**
     * 获得部门客户分页
     *
     * @param pageReqVO 分页查询
     * @return 客户分页
     */
    PageResult<CustomerDO> getDeptCustomerPage(CustomerPageReqVO pageReqVO);

    PageResult<CustomerDO> getDeptCustomerPageReq(CustomerPageReqDTO pageReqVO  ,PageVO page);

    /**
     * 获得部门客户分页
     *
     * @param exportReqVO 查询条件
     * @return 客户列表
     */
    List<CustomerDO> getDeptCustomerExcelVoList(CustomerExportReqVO exportReqVO);

    List<CustomerDO> getDeptCustomerExcelVoListReq(CustomerExportReqDTO exportReqVO);

    /**
     * 获得待接收客户分页
     *
     * @param pageReqVO 分页查询
     * @return 客户分页
     */
    PageResult<CustomerDO> getWaitToConfirmCustomerPage(CustomerPageReqVO pageReqVO);

    PageResult<CustomerDO> getWaitToConfirmCustomerPageReq(CustomerPageReqDTO pageReqVO,PageVO page);
    /**
     * "我的待接收客户列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 客户列表
     */
    List<CustomerDO> getWaitToConfirmCustomerExcelVoList(CustomerExportReqVO exportReqVO);

    /**
     * 我的待接收客户列表, 用于 Excel 导出 ,支持多选
     * @param exportReqVO
     * @return
     */
    List<CustomerDO> getWaitToConfirmCustomerExcelVoListReq(CustomerExportReqDTO exportReqVO);
    /**
     * 获得待分配客户分页
     *
     * @param pageReqVO 分页查询
     * @return 客户分页
     */
    PageResult<CustomerDO> getWaitToAssignedCustomerPage(CustomerPageReqVO pageReqVO);


    /**
     * 获得待分配客户分页 支持多选
     * @param pageReqVO
     * @return
     */
    PageResult<CustomerDO> getWaitToAssignedCustomerPageReq(CustomerPageReqDTO pageReqVO,PageVO page);

    /**
     * 待分配客户列表导出, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 客户列表
     */
    List<CustomerDO> getWaitToAssignedCustomerExcelVoList(CustomerExportReqVO exportReqVO);

    /**
     * 待分配客户列表导出, 用于 Excel 导出 支持多筛选
     * @param exportReqVO
     * @return
     */
    List<CustomerDO> getWaitToAssignedCustomerExcelVoListReq(CustomerExportReqDTO exportReqVO);
    /**
     * 获得公海池客户分页
     *
     * @param pageReqVO 分页查询
     * @return 客户分页
     */
    PageResult<CustomerDO> getOpenSeaCustomerPage(CustomerPageReqVO pageReqVO);

    PageResult<CustomerDO> getOpenSeaCustomerPageReq(CustomerPageReqDTO pageReqVO,PageVO page);

    /**
     * 获得公海池客户列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 客户列表
     */
    List<CustomerDO> getPublicCustomerExcelVoList(CustomerExportReqVO exportReqVO);

    List<CustomerDO> getPublicCustomerExcelVoReqList(CustomerExportReqDTO exportReqVO);

    /**
     * 获得客户列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 客户列表
     */
    List<CustomerDO> getCustomerExcelVoList(CustomerExportReqVO exportReqVO);

    List<CustomerDO> getMyCustomerExcelVoList(CustomerExportReqDTO exportReqVO);

    /**
     * 获得客户列表, 用于 Excel 导出
     *
     * @param customerSelectReqVO 查询条件
     * @return 客户列表
     */
    PageResult<CustomerDO> searchCustomer(CustomerSelectReqVO customerSelectReqVO);


    /**
     * 根据登录客户的id找到后台对应的ecw_customer数据
     *
     * @param memberUserId
     * @return
     */
    CustomerDO getByMemberUserId(Long memberUserId);


    /**
     * 根据客户ID更新公司名称
     *
     * @return
     */
    void asyncUpdateCompany(Long customerId, String company);


    void updateCustomerStatus(Long customerId, CustomerStatusEnum customerStatusEnum);


    CustomerDelayApprovalInfoDto getDelayApprovalInfo(Long customerId);

    CustomerDelayApprovalInfoDto getDelayApproval(Long approvalId);

    void delayApproval(CustomerDelayApprovalInfoDto delayApprovalInfoDto);

    /**
     * 取消延期申请
     *
     * @param approveId 业务ID
     * @return 取消延期申请
     */
    void cancelDelayApproval(Long approveId, String reason);


    /**
     * 定时任务批量客户进入公海池时判断是否有在审批中的延期申请，是就取消延期申请
     *
     * @param customerIds 客户id列表
     * @param reason 取消原因
     * @return 取消延期申请
     */
    void batchCancelDelayApproval(List<Long> customerIds, String reason);


    /**
     * @param approveId 表单业务ID
     * @param result    1 处理中  2 通过 3 不通过 4 已取消
     */
    void approvalCustomerDelay(String approveId, Integer result);
    void approvalCustomerTransfer(String approveId, Integer result);

    // CustomerHandoverApprovalInfoDto getHandoverApprovalInfo(Long customerId);

    CustomerHandoverApprovalInfoDto getHandoverApproval(Long approvalId);

    String handoverApproval(CustomerHandOverReqVO customerHandOverReqVO);
    void cancelHandoverApproval(Long approveId, String reason);

    /**
     * @param hours 小时参数
     */
    void moveWaitConfirmUserToWaitAssigned(Integer hours);
    void moveWebConsigneeCustomerToWaitAssigned(Integer hours);

    /**
     * @param var1 时间参数1
     * @param var2 时间参数2
     * @param var3 时间参数3
     */
    void scheduleCustomerRefresh2(Integer var1, Integer var2, Integer var3);


    /**
     *
     *<p>
     * 获取小于时间限制的可进入公海池的客户列表。
     *</p>
     *
     *<p>
     *     当时间限制为0时，就需要进入公海池列表。 同时满足界面查询，比如查10天后进入公海池的数据，可以展示。也可以满足将进入公海池的用户做纪录
     *</p>
     *
     * @param limitDay 小于的天
     *
     * @return
     */
    List<CustomerDO> getToOpenSeaTimeAfterLimitDayCustomerList(@Param("limitDay") Integer limitDay);


    /**
     * 分配客户经理
     *
     * @param customerDOList
     * @param customerServiceId
     * @param customerOperateTypeEnum
     * @param reason
     * @param reqBody
     */
    void assignedCustomer(List<CustomerDO> customerDOList, Long customerServiceId, CustomerOperateTypeEnum customerOperateTypeEnum, String reason, String reqBody);


    void confirmCustomer(Long customerId);


    /**
     * 捞取客户
     * @param customerId
     */
    void catchCustomer(Long customerId);

    /**
     * 接收客户
     *
     * @param customerId
     * @param customerServiceId
     */
    void receiveCustomer(Long customerId, Long customerServiceId);


    /**
     * 报价后更新客户
     *
     * @param customerId      客户id
     * @param offerCreateTime
     */
    void updateCustomerByCreateOffer(Long customerId, Date offerCreateTime);

    /**
     * 订单创建后更新客户信息
     *
     * @param customerId
     * @param orderId
     * @param orderNo
     * @param orderCreateTime
     */
    void updateCustomerByCreateOrder(Long customerId, Long orderId, String orderNo, Date orderCreateTime);


    /**
     * 入仓完成后更新客户信息
     *
     * @param customerId
     * @param orderId
     * @param orderNo
     * @param rucangTime
     */
    void updateCustomerByFinishWarehouseIn(Long customerId, Long orderId, String orderNo, Date rucangTime);


    /**
     * 更新入公海时间和状态
     *
     * @param customerId                 客户id
     * @param customerStatusEnum         客户状态
     * @param estimateEnterToOpenSeaTime
     * @param logOrNot                   是否纪录日志
     * @param logRemark
     */
    void updateCustomerStatusAndEstimateEnterOpenSeaTime(Long customerId, CustomerStatusEnum customerStatusEnum, Date estimateEnterToOpenSeaTime, boolean logOrNot, String logRemark);

    /**
     * 根据现有的订单数据和报价数据 更新客户状态
     *
     * @param customerId 客户id
     * @param logOrNot 是否纪录日志
     */
    void updateCustomerStatusByExistedOrderAndOffer(Long customerId, boolean logOrNot);


    void enterToOpenSea(List<CustomerDO> customerDOList);

    void changeOldCustomerNumber();


    /**
     * 客户切换空运/非空运
     *
     * @param customerChangeAirReqVO
     */
    void changeCustomerAir(CustomerChangeAirReqVO customerChangeAirReqVO);


    /**
     * 客户切换海运整柜/非海运整柜
     *
     * @param customerChangeFclReqVO
     */
    void changeCustomerFcl(CustomerChangeFclReqVO customerChangeFclReqVO);

    /**
     * @param areaCode          区号  eg: 86
     * @param phone             电话  eg: 13233334444
     * @param excludeCustomerId 需要排除的客户id。目的是判断此客户外还有没有其他人拥有此联系人。更新时就需要使用这个字段
     * @return
     */
    CustomerContactsDO getUniqueOneAndValidate(String areaCode, String phone, Long excludeCustomerId);


    /**
     * 是否空运或海运客户
     * @param customer
     * @return
     */
    boolean isFcl(CustomerDO customer);

    /**
     * 回收已分配未确认的客户
     *
     * @param customerUnConfirmReqVO 已分配未确认的客户id
     */
    void recycleUnConfirmedCustomer(CustomerUnConfirmReqVO customerUnConfirmReqVO);

    /**
     * 将客户经理下已成交的客户全部修改为老客户
     *
     * @param customerService 客户经理id
     * @return 更新的客户id集合
     */
    List<Long> updateCustomerToOldByCustomerService(Long customerService);
    void asyncLogUpdateCustomerToOldByCustomerService(Long customerService, List<Long> newDealCustomerIdList);

    /**
     * 根据客户id更新客户的新老类型
     *
     * @param customerId 客户id
     * @param isNew 新客户=true,老客户=false
     */
    void updateCustomerNewOrOld(Long customerId, Boolean isNew);
}
